
import os.path
from enum import Enum, unique
from urllib.parse import urljoin

from requests import Session
from requests.models import Response


class _HTTPMethod:
    GET = "GET"
    POST = "POST"


@unique
class Request(Enum):
    HealthCheck = "health_check"
    PullDatasetByName = "pull_dataset_by_name"


class _URLBuilder:
    _API_VERSION = "v1"

    def __init__(self) -> None:
        self._common_request_prefix = "/api/" + self._API_VERSION + "/"
        self._dataset_request_prefix = self._common_request_prefix + "datasets/"

        self._builder = {
            Request.HealthCheck: self._health_check,
            Request.PullDatasetByName: self._pull_dataset_by_name
        }

    @staticmethod
    def _health_check(**kwargs) -> str:
        return "/health/check/"

    def _pull_dataset_by_name(self, **kwargs) -> str:
        return urljoin(self._dataset_request_prefix, os.path.join(kwargs["dataset"], "files/"))

    def build(self, req: Request, **kwargs) -> str:
        return self._builder[req](**kwargs)

    def version(self) -> str:
        return self._API_VERSION


class Client:
    _HOME = "http://192.168.20.41:8080/"

    def __init__(self) -> None:
        self._url_builder = _URLBuilder()

    def _make_url(self, req: Request, **kwargs) -> str:
        return self._url_builder.build(req, **kwargs)

    def do(self, req: Request, **kwargs) -> Response:
        url = urljoin(self._HOME, self._make_url(req, **kwargs))
        resp = Session().request(method=_HTTPMethod.GET, url=url)
        return resp



import click

from odi import __version__


@click.group(context_settings={"help_option_names": ("-h", "--help")})
@click.pass_context
def cli(ctx: click.Context) -> None:
    pass


@cli.command()
@click.pass_obj
def auth(ctx: str) -> None:
    print("login")


@cli.command()
@click.pass_obj
def init(ctx: str) -> None:
    print("init")


@cli.command()
@click.argument("dataset", type=str)
@click.option(
    "-p", "--path", type=str, default="", help="Path to pull the dataset"
)
@click.pass_obj
def pull(ctx: str, dataset: str, path: str) -> None:
    from odi.cli.command.pull import implement_pull
    implement_pull(dataset, path)


@cli.command()
@click.pass_obj
def push(ctx: str) -> None:
    print("get")


@cli.command()
@click.pass_obj
def search(ctx: str) -> None:
    print("search")


@cli.command()
@click.pass_obj
def version(ctx: str) -> None:
    print(__version__)


if __name__ == "__main__":
    cli(prog_name="odi")



import os.path
from concurrent.futures import FIRST_EXCEPTION, ThreadPoolExecutor, wait
from warnings import warn

from odi.client.request import Client, Request
from odi.client.storage import S3, S3Object


class ODI:
    def __init__(self):
        self._client = Client()

    def pull(self, *, dataset: str, path: str = "") -> bool:
        if path and not os.path.exists(path):
            print("No such path.")
            return False

        resp = self._client.do(Request.PullDatasetByName, dataset=dataset).json()
        if not resp["success"] or resp["httpCode"] != 200:
            print(resp["message"])
            return False

        data = resp["data"]
        files = data["files"]
        if not files:
            print(f"Cannot find the dataset [{dataset}].")
            return False

        if data["truncated"]:
            warn(f"Dataset [{dataset}] is truncated.")

        s3objs = []
        for f in files:
            s3objs.append(S3Object(f["name"], f["fullPath"], f["size"], f["url"]))
        print(f"Pulling {dataset} (Total: {len(s3objs)})...")
        self._download_from_storage(s3objs, path)

    def _download_from_storage(self, files, path: str):
        max_workers = len(files)
        with ThreadPoolExecutor(max_workers) as executor:
            futures = [executor.submit(S3.download_obj, file, pos, path) for pos, file in enumerate(files)]

            done, not_done = wait(futures, return_when=FIRST_EXCEPTION)



import os

import requests
from tqdm import tqdm


class S3Bucket:
    def __init__(self, name: str) -> None:
        self.name = name


class S3Object:
    def __init__(self, name: str, path: str, size: int, url: str) -> None:
        self.name = name
        self.path = path
        self.size = size
        self.url = url


class S3:
    def __init__(self) -> None:
        pass

    @staticmethod
    def download_obj(obj: S3Object, position: int, path: str = "./"):
        block_size = 1024
        response = requests.get(obj.url, stream=True)
        with tqdm(
                desc=obj.path,
                total=obj.size,
                unit="iB",
                unit_scale=True,
                leave=None,
                position=position
        ) as bar:
            file_path = os.path.join(path, obj.path)
            if not os.path.exists(os.path.dirname(file_path)):
                os.makedirs(os.path.dirname(file_path))
            with open(file_path, "wb") as file:
                for data in response.iter_content(block_size):
                    bar.update(len(data))
                    file.write(data)



from typing import Optional


class ODIException(Exception):
    def __init__(self, message: Optional[str] = None) -> None:
        super().__init__()
        self._message = message

    def __str__(self) -> str:
        return self._message if self._message else ""


class DatabaseError(ODIException):
    def __init__(self, message: Optional[str] = None) -> None:
        super().__init__(message)
